/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2018                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2015                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2011                                                     **
**  University of Oregon, Eugene, USA                                      **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



package scalasca.cubex.cube.services;
import java.lang.String;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import scalasca.cubex.cube.*;
import scalasca.cubex.cube.errors.*;


/*
 * Class , delivers file structure of CubeX
 *
 */
public class CnodesEnumeration
{
    protected String                                  name        = "Trivial Enumeration (1->1, 2->2, ...)";
    protected Map<scalasca.cubex.cube.Cnode, Integer> enumeration = new HashMap<scalasca.cubex.cube.Cnode, Integer>();     // this might be replaced by ArrayList

    public CnodesEnumeration()
    {
    }

    public CnodesEnumeration( ArrayList<scalasca.cubex.cube.Cnode> root_cnodes )
    {
        enumerate( root_cnodes );
    }


    public int
    getCID( scalasca.cubex.cube.Cnode cnode ) throws NotEnumeratedCnodeException
    {
        Integer CID = enumeration.get( cnode );
        if ( CID == null )
        {
            throw new NotEnumeratedCnodeException( "Something is wrong. Tried to get a CID of not enumerated Cnode" );
        }
        return CID.intValue();
    }



    protected void
    enumerate( ArrayList<scalasca.cubex.cube.Cnode> root_cnodes )
    {
        ArrayList<scalasca.cubex.cube.Cnode> _cnodes = make_list( root_cnodes );
        for ( int i = 0; i < _cnodes.size(); i++ )
        {
            enumeration.put( _cnodes.get( i ), new Integer( _cnodes.get( i ).getId() ) );
        }
    }


    protected ArrayList<scalasca.cubex.cube.Cnode>
    make_list( ArrayList<scalasca.cubex.cube.Cnode> root_cnodes )
    {
        ArrayList<scalasca.cubex.cube.Cnode> _list = new ArrayList<scalasca.cubex.cube.Cnode>();

        for ( scalasca.cubex.cube.Cnode cnode: root_cnodes )
        {
            addToList( _list, cnode );
        }
        return _list;
    }


    private void
    addToList( ArrayList<scalasca.cubex.cube.Cnode> _list, scalasca.cubex.cube.Cnode cnode )
    {
        _list.add( cnode );
        for ( scalasca.cubex.cube.Vertex _cnode : cnode.getAllChildren() )
        {
            addToList( _list, ( scalasca.cubex.cube.Cnode )_cnode );
        }
    }
}
