/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2018                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2015                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



package scalasca.cubex.tarviewer;

import java.io.IOException;
import java.io.InputStream;

import org.xeustechnologies.jtar.TarEntry;
import org.xeustechnologies.jtar.TarHeader;
import org.xeustechnologies.jtar.TarInputStream;


/**

   Provides for TarInputStream a list of files in the tar file with size, offset and a name.
   Used by Cube to get a direct access to the tar content.
 */
public class BTarInputStream extends TarInputStream
{
    private long chunkCnt;

    public BTarInputStream( InputStream in )
    {
        super( in );
        chunkCnt = 0;
    }

    /**
     * Gets the next entry inside the TAR archive
     * @return BTarEntry object representing the next entry inside the TAR archive, NULL otherwise
     * @throws IOException
     */
    public BTarEntry
    next() throws IOException
    {
        TarEntry te = super.getNextEntry();
        chunkCnt++;
        if ( te == null )
        {
            return null;
        }
        if ( te.getLinkFlag() == TarHeader.LF_LPAX ) // extended pax header
        {
            long   size      = te.getSize();
            byte[] pax_block = new byte[ ( int )size ];
            super.read( pax_block, 0, ( int )size );
            PaxEntry pax = new PaxEntry( pax_block );
            chunkCnt += size / 512 + ( size % 512 == 0 ? 0 : 1 );
            te        = super.getNextEntry();
            te.setSize( pax.getSize() );
            chunkCnt++;
        }
        long      size = te.getSize();
        BTarEntry bte  = new BTarEntry( te.getName(), size, chunkCnt * 512, te.isDirectory() );
        chunkCnt += size / 512 + ( size % 512 == 0 ? 0 : 1 );
        return bte;
    }
}
